using System;
using System.Collections.Generic;
using System.Text;
using Syncfusion.Windows.Forms.Grid;
using System.Runtime.Serialization;
using System.Drawing;
using System.Windows.Forms;
using System.Data;

namespace WindowsApplication24 {
    public class MyComboBoxCellModel : GridDropDownCellModel {
        public MyComboBoxCellModel(GridModel grid)
            : base(grid) {
        }

        protected MyComboBoxCellModel(SerializationInfo info, StreamingContext context)
            : base(info, context) {
        }

        public override GridCellRendererBase CreateRenderer(GridControlBase control) {
            return new MyComboBoxCellRenderer(control, this);
        }
    }

    public class MyComboBoxCellRenderer : GridDropDownCellRenderer {
        private System.Windows.Forms.ComboBox comboBox;
        private static string fakeControlText = "-1";
        private bool textChanging = false;
        private bool controlKeyPressed;


        public MyComboBoxCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
            : base(grid, cellModel) {
            this.comboBox = new ComboBox();
            this.comboBox.Name = "CB_EDIT";
            this.comboBox.Visible = false;
            DropDownButton = new GridCellComboBoxButton(this);
            grid.Controls.Add(this.comboBox);
            this.TextBox.TextChanged += new EventHandler(TextBox_TextChanged);
            this.TextBox.KeyDown += new KeyEventHandler(TextBox_KeyDown);
        }

        #region Stuff responsible for editing the cell

        void TextBox_KeyDown(object sender, KeyEventArgs e) {
            if (e.KeyValue == 8 || e.KeyValue == 46) {
                this.controlKeyPressed = true;
            } else {
                this.controlKeyPressed = false;
            }
        }

        void TextBox_TextChanged(object sender, EventArgs e) {
            if (!textChanging && Grid.CurrentCell.ColIndex == ColIndex && Grid.CurrentCell.RowIndex == RowIndex) {
                textChanging = true;
                string tekst = this.TextBox.Text;
                if (!String.IsNullOrEmpty(tekst)) {
                    ComboBox gc = this.comboBox;
                    int i = gc.FindString(tekst);
                    if (StyleInfo.DropDownStyle == GridDropDownStyle.Editable) {
                        gc.SelectedIndex = i;
                        int j = i;
                        while (i > 0 && (!(gc.Text.StartsWith(tekst)))) {
                            j = gc.FindString(tekst, i);
                            if (j <= i) {
                                i = -1;
                            } else {
                                i = j;
                            }
                            gc.SelectedIndex = i;
                        }
                    }
                    if (i > -1) {
                        gc.SelectedIndex = i;
                        if (StyleInfo.DropDownStyle != GridDropDownStyle.Editable) {
                            this.TextBox.Text = gc.Text;
                            if (!controlKeyPressed) {
                                this.TextBox.SelectionStart = tekst.Length;
                                this.TextBox.SelectionLength = this.TextBox.Text.Length - tekst.Length;
                            } else {
                                this.TextBox.SelectionStart = tekst.Length - 1;
                                this.TextBox.SelectionLength = this.TextBox.Text.Length - tekst.Length + 1;
                            }
                        } else {
                            if (!controlKeyPressed) {
                                this.TextBox.Text = gc.Text;
                                this.TextBox.SelectionStart = tekst.Length;
                                this.TextBox.SelectionLength = this.TextBox.Text.Length - tekst.Length;
                            } else {
                                this.TextBox.SelectionStart = tekst.Length;
                                this.TextBox.SelectionLength = 0;
                            }

                        }
                    } else {
                    }
                }
                textChanging = false;
            }
        }

        #endregion

        #region usual renderer overrides

        /// <summary>
        /// I do not what DropDownContainer to show up
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public override void DropDownContainerShowingDropDown(object sender, System.ComponentModel.CancelEventArgs e) {
            e.Cancel = true;
            //base.DropDownContainerShowingDropDown(sender, e);
        }

        /*
        public override bool ValidateString(string text) {
            string validaTedString = text;
            if (text == fakeeControlText) {
                validaTedString = this.TextBox.Text;
            }
            if (StyleInfo.DropDownStyle != GridDropDownStyle.Editable) {
                ComboBox gc = this.comboBox;
                int i = gc.FindString(validaTedString);
                if (i == -1) {
                    return false;
                }
            }
            return true;
        }
        */

        /// <summary>
        /// Responsible for drawing display member instead of value member
        /// </summary>
        protected override void OnDraw(System.Drawing.Graphics g, System.Drawing.Rectangle clientRectangle, int rowIndex, int colIndex, Syncfusion.Windows.Forms.Grid.GridStyleInfo style) {
            bool gridInEditMode = Grid.CurrentCell.IsEditing;
            this.comboBox.DisplayMember = style.DisplayMember;
            this.comboBox.ValueMember = style.ValueMember;
            this.comboBox.DataSource = style.DataSource;
            bool currentCombo = (this.RowIndex == Grid.CurrentCell.RowIndex) && (this.ColIndex == Grid.CurrentCell.ColIndex);
            if (Grid.CurrentCell.HasCurrentCellAt(rowIndex, colIndex) && gridInEditMode && currentCombo) {
                base.OnDraw(g, clientRectangle, rowIndex, colIndex, style);
            } else {
                if (style.DropDownStyle != GridDropDownStyle.Editable) {
                    style.CellValueType = typeof(string);
                    
                    string text = style.CellValue.ToString();
                    if (style.DataSource != null) {
                        DataTable dt = (style.DataSource as DataTable);
                        if (!String.IsNullOrEmpty(style.Text)) {
                            dt.DefaultView.Sort = style.ValueMember;
                            int rv = dt.DefaultView.Find(style.Text);
                            if (rv > -1) {
                                DataRowView drv = dt.DefaultView[rv];
                                style.Text = drv[style.DisplayMember].ToString();
                            } else {
                                style.Text = String.Empty;
                            }
                        }
                    }
                }
                base.OnDraw(g, clientRectangle, rowIndex, colIndex, style);
            }
        }

        public override object ControlValue {
            get {
                GridStyleInfo style = Grid.Model[this.RowIndex, this.ColIndex];
                    ComboBox gcb = this.comboBox;
                    string t = this.TextBox.Text;
                    if (StyleInfo.DropDownStyle == GridDropDownStyle.Editable) {
                        string s = t.Trim();
                        if (String.IsNullOrEmpty(s)) {
                            return DBNull.Value;
                        } else {
                            return s;
                        }
                    } else {
                        int i = gcb.FindStringExact(t);
                        if (i > -1) {
                            if (gcb.SelectedValue == null || String.IsNullOrEmpty(t.Trim())) {
                                style.CellValue = DBNull.Value;
                            } else {
                                if (base.ControlValue != null) {
                                    gcb.SelectedValue = base.ControlValue;
                                } else {
                                    gcb.SelectedValue = DBNull.Value;
                                }
                                if (gcb.Text == t) {
                                    return gcb.SelectedValue;
                                }
                                gcb.SelectedIndex = i;
                                return gcb.SelectedValue;
                            }
                        }
                    }
                return DBNull.Value;
            }
            set {
                base.ControlValue = value;
                if (value != null && String.IsNullOrEmpty(value.ToString())) {
                    this.TextBox.Text = String.Empty;
                } else {
                    ComboBox gcb = this.comboBox;
                    gcb.SelectedValue = value;
                    this.TextBox.Text = gcb.Text;
                }
            }
        }

        public override string ControlText {
            get {
                if (StyleInfo.DropDownStyle != GridDropDownStyle.Editable) {
                    if (String.IsNullOrEmpty(this.TextBox.Text)) {
                        return String.Empty;
                    } else {
                        //return fakeControlText;
                        return this.TextBox.Text;

                    }
                } else {
                    return this.TextBox.Text;
                }
            }
            set {
                this.textChanging = true;
                try {
                    if (String.IsNullOrEmpty(value)) {
                        base.ControlText = value;
                        return;
                    }
                    if (StyleInfo.DropDownStyle != GridDropDownStyle.Editable) {
                        long l;
                        if (Int64.TryParse(value, out l)) {
                            if (ControlValue != null) {
                                if (ControlValue.ToString() == value) {
                                    ComboBox gc = this.comboBox;
                                    gc.SelectedValue = l;
                                    base.ControlText = gc.Text;
                                    return;
                                }
                            }
                        } else {
                            base.ControlText = String.Empty;
                        }
                    }
                    base.ControlText = value;
                } finally {
                    this.textChanging = false;
                }
            }
        }

        protected override bool OnSaveChanges() {
            GridStyleInfo style = Grid.Model[this.RowIndex, this.ColIndex];
            if (CurrentCell.IsModified) {
                ComboBox gcb = this.comboBox;
                string t = this.TextBox.Text;
                if (style.DropDownStyle == GridDropDownStyle.Editable) {
                    string s = t.Trim();
                    if (String.IsNullOrEmpty(s)) {
                        style.CellValue = DBNull.Value;
                    } else {
                        style.CellValue = s;
                    }
                    return true;
                } else {
                    int i = gcb.FindStringExact(t);
                    if (i > -1) {
                        if (gcb.SelectedValue == null || String.IsNullOrEmpty(t.Trim())) {
                            style.CellValue = DBNull.Value;
                        } else {
                            gcb.SelectedValue = this.ControlValue;
                            if (gcb.Text == t) {
                                style.CellValue = gcb.SelectedValue;
                                this.ControlValue = style.CellValue;
                                return true;
                            }
                            gcb.SelectedIndex = i;
                            style.CellValue = gcb.SelectedValue;
                            this.ControlValue = style.CellValue;
                        }
                        return true;
                    }
                }
            }
            return false;

        }

        /*
        protected override void OnInitialize(int rowIndex, int colIndex) {
            this.ControlValue = Grid.Model[rowIndex, colIndex].CellValue;
            base.OnInitialize(rowIndex, colIndex);
        }
        */
        protected override bool OnValidate()
        {
            return this.OnSaveChanges(); 
        }

        #endregion
    }
}
